/*
 *   Copyright (C) 2009-2014,2016,2018,2020 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	KenwoodNetwork_H
#define	KenwoodNetwork_H

#include "UDPSocket.h"
#include "Timer.h"

#include <cstdint>
#include <string>
#include <random>

class CKenwoodNetwork {
public:
	CKenwoodNetwork(const std::string& address, bool debug);
	~CKenwoodNetwork();

	bool open();

	bool write(const unsigned char* data, unsigned int length);

	unsigned int read(unsigned char* data);

	void close();

	void clock(unsigned int ms);

private:
    CUDPSocket       m_rtpSocket;
    CUDPSocket       m_rtcpSocket;
    sockaddr_storage m_rtpAddr;
    unsigned int     m_rtpAddrLen;
    sockaddr_storage m_rtcpAddr;
    unsigned int     m_rtcpAddrLen;
    bool             m_headerSeen;
    bool             m_seen1;
    bool             m_seen2;
    bool             m_seen3;
    bool             m_seen4;
    unsigned char*   m_sacch;
    uint8_t          m_sessionId;
    uint16_t         m_seqNo;
    unsigned int     m_ssrc;
    bool             m_debug;
    uint32_t         m_startSecs;
    uint32_t         m_startUSecs;
    CTimer           m_rtcpTimer;
    CTimer           m_hangTimer;
    unsigned char    m_hangType;
    unsigned short   m_hangSrc;
    unsigned short   m_hangDst;
    std::mt19937     m_random;

    bool processIcomVoiceHeader(const unsigned char* data);
    bool processIcomVoiceData(const unsigned char* data);
    unsigned int processKenwoodVoiceHeader(unsigned char* data);
    unsigned int processKenwoodVoiceData(unsigned char* data);
    unsigned int processKenwoodVoiceLateEntry(unsigned char* data);
    unsigned int processKenwoodData(unsigned char* data);
    bool writeRTPVoiceHeader(const unsigned char* data);
    bool writeRTPVoiceData(const unsigned char* data);
    bool writeRTPVoiceTrailer(const unsigned char* data);
    bool writeRTCPStart();
    bool writeRTCPPing();
    bool writeRTCPHang(unsigned char type, unsigned short src, unsigned short dst);
    bool writeRTCPHang();
    unsigned int readRTP(unsigned char* data);
    unsigned int readRTCP(unsigned char* data);
    unsigned long getTimeStamp() const;
};

#endif
